@@ -7,9 +7,9 @@
 #include "libcman.h"
 #include "cman_tool.h"
 
-#define DEFAULT_CONFIG_MODULE "ccsconfig"
+#define DEFAULT_CONFIG_MODULE "xmlconfig"
 
-#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:k:F:C:VPwfqah?Xd::")
+#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:k:F:C:VAPwfqah?Xd::")
 #define OP_JOIN			1
 #define OP_LEAVE		2
 #define OP_EXPECTED		3
@@ -50,13 +50,14 @@ static void print_usage(int subcmd)
 		printf("  -n <nodename>    The name of this node (defaults to hostname)\n");
 		printf("  -c <clustername> The name of the cluster to join\n");
 		printf("  -N <id>          Node id\n");
-		printf("  -C <module>      Config file reader (default: ccsconfig)\n");
+		printf("  -C <module>      Config file reader (default: " DEFAULT_CONFIG_MODULE ")\n");
 		printf("  -w               Wait until node has joined a cluster\n");
 		printf("  -q               Wait until the cluster is quorate\n");
 		printf("  -t               Maximum time (in seconds) to wait\n");
 		printf("  -k <file>        Private key file for AIS communications\n");
 		printf("  -P               Don't set aisexec to realtime priority\n");
 		printf("  -X               Use internal cman defaults for configuration\n");
+		printf("  -A               Don't load openais services\n");
 		printf("\n");
 	}
 
@@ -862,6 +863,10 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			comline->noconfig_opt = TRUE;
 			break;
 
+		case 'A':
+			comline->noopenais_opt = TRUE;
+			break;
+
 		case 'P':
 			comline->nosetpri_opt = TRUE;
 			break;
