@@ -1,7 +1,6 @@
 #!/usr/bin/python
 
-##
-## Copyright (C) 2008 Red Hat, Inc. All Rights Reserved.
+#####
 ##
 ## The Following Agent Has Been Tested On:
 ##
@@ -36,8 +35,7 @@ def get_power_status(conn, options):
 	## Transformation to standard ON/OFF status if possible
 	if status == "Running":
 		status = "on"
-
-	if status == "Not Activated":
+	else:
 		status = "off"
 
 	return status
