@@ -577,7 +577,7 @@ static void *process_queries(void *arg)
 			goto out;
 		}
 
-		pthread_mutex_lock(&query_mutex);
+		query_lock();
 
 		switch (h.command) {
 		case FENCED_CMD_DUMP_DEBUG:
@@ -595,7 +595,7 @@ static void *process_queries(void *arg)
 		default:
 			break;
 		}
-		pthread_mutex_unlock(&query_mutex);
+		query_unlock();
 
  out:
 		close(f);
@@ -793,7 +793,7 @@ static void loop(void)
 			goto out;
 		}
 
-		pthread_mutex_lock(&query_mutex);
+		query_lock();
 
 		for (i = 0; i <= client_maxi; i++) {
 			if (client[i].fd < 0)
@@ -807,7 +807,7 @@ static void loop(void)
 				deadfn(i);
 			}
 		}
-		pthread_mutex_unlock(&query_mutex);
+		query_unlock();
 
 		if (daemon_quit)
 			break;
@@ -1012,10 +1012,7 @@ int main(int argc, char **argv)
 	INIT_LIST_HEAD(&domains);
 	INIT_LIST_HEAD(&controlled_entries);
 
-	init_logging();
-
 	read_arguments(argc, argv);
-
 	lockfile();
 
 	if (!daemon_debug_opt) {
@@ -1025,8 +1022,8 @@ int main(int argc, char **argv)
 		}
 		umask(0);
 	}
+	init_logging();
 	signal(SIGTERM, sigterm_handler);
-
 	set_oom_adj(-16);
 
 	loop();
