@@ -212,6 +212,7 @@ static int init_system_inodes(struct gfs2_sbd *sdp)
 
 	gfs2_lookupi(sdp->master_dir, "statfs", 6, &sdp->md.statfs);
 	buf = malloc(sdp->md.statfs->i_di.di_size);
+	// FIXME: handle failed malloc
 	gfs2_readi(sdp->md.statfs, buf, 0, sdp->md.statfs->i_di.di_size);
 	/* call gfs2_inum_range_in() to retrieve range */
 	gfs2_statfs_change_in(&sc, buf);
@@ -295,11 +296,11 @@ static int fill_super_block(struct gfs2_sbd *sdp)
 	 ********************************************************************/
 	log_info("Initializing lists...\n");
 	osi_list_init(&sdp->rglist);
-	init_buf_list(sdp, &sdp->buf_list, 128 << 20);
-	init_buf_list(sdp, &sdp->nvbuf_list, 0xffffffff);
+	osi_list_init(&sdp->buf_list);
 	for(i = 0; i < BUF_HASH_SIZE; i++) {
 		osi_list_init(&dir_hash[i]);
 		osi_list_init(&inode_hash[i]);
+		osi_list_init(&sdp->buf_hash[i]);
 	}
 
 	/********************************************************************
