@@ -67,6 +67,7 @@ int read_sb(struct gfs2_sbd *sdp)
 		goto out;
 
 	sdp->sd_fsb2bb_shift = sdp->sd_sb.sb_bsize_shift - GFS2_BASIC_BLOCK_SHIFT;
+	sdp->bsize = sdp->sd_sb.sb_bsize;
 	sdp->sd_diptrs =
 		(sdp->sd_sb.sb_bsize-sizeof(struct gfs2_dinode)) /
 		sizeof(uint64_t);
@@ -183,6 +184,7 @@ int rindex_read(struct gfs2_sbd *sdp, int fd, int *count1)
 			return -1;
 
 		rgd = (struct rgrp_list *)malloc(sizeof(struct rgrp_list));
+		// FIXME: handle failed malloc
 		memset(rgd, 0, sizeof(struct rgrp_list));
 		osi_list_add_prev(&rgd->list, &sdp->rglist);
 
