@@ -67,6 +67,7 @@
 
 extern int daemon_debug_opt;
 extern int daemon_quit;
+extern int cluster_down;
 extern int poll_fencing;
 extern int poll_quorum;
 extern int poll_fs;
@@ -127,7 +128,8 @@ do { \
 
 /* dlm_header types */
 enum {
-	DLM_MSG_START = 1,
+	DLM_MSG_PROTOCOL = 1,
+	DLM_MSG_START,
 	DLM_MSG_PLOCK,
 	DLM_MSG_PLOCK_OWN,
 	DLM_MSG_PLOCK_DROP,
@@ -234,6 +236,9 @@ int get_weight(int nodeid, char *lockspace);
 
 /* cpg.c */
 int setup_cpg(void);
+void close_cpg(void);
+void process_cpg(int ci);
+int set_protocol(void);
 void process_lockspace_changes(void);
 void dlm_send_message(struct lockspace *ls, char *buf, int len);
 int dlm_join_lockspace(struct lockspace *ls);
