@@ -1,15 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2008 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include "gfs_daemon.h"
 #include "config.h"
 #include "libdlmcontrol.h"
@@ -1418,10 +1406,6 @@ static void receive_first_recovery_done(struct mountgroup *mg,
 		  "mount_client_notified %d",
 		  hd->nodeid, master, mg->mount_client_notified);
 
-	if (master != hd->nodeid)
-		log_error("receive_first_recovery_done from %d master %d",
-			  hd->nodeid, master);
-
 	if (list_empty(&mg->changes)) {
 		/* everything is idle, no changes in progress */
 
@@ -1453,6 +1437,12 @@ static void receive_first_recovery_done(struct mountgroup *mg,
 		     start messages either before or after receiving this
 		     message). */
 
+		/* exclude new nodes from this sanity check since they've
+		   never set a master value to compare against */
+		if (mg->started_count && (master != hd->nodeid))
+			log_error("receive_first_recovery_done from %d "
+				  "master %d", hd->nodeid, master);
+
 		mg->first_recovery_needed = 0;
 		mg->first_recovery_master = 0;
 		mg->first_recovery_msg = 1;
