@@ -1,21 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2006
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 #define _MESSAGE_BUILD
 #include <message.h>
 #include <stdio.h>
@@ -33,7 +15,7 @@
 #include <signals.h>
 #include <gettid.h>
 #include <cman-private.h>
-#include <clulog.h>
+#include <logging.h>
 
 /* Ripped from ccsd's setup_local_socket */
 
@@ -727,7 +709,7 @@ queue_for_context(msgctx_t *ctx, char *buf, int len)
 	msg_q_t *node;
 
 	if (ctx->type != MSG_CLUSTER) {
-		clulog(LOG_WARNING, "%s called on invalid context %p\n",
+		log_printf(LOG_WARNING, "%s called on invalid context %p\n",
 		       __FUNCTION__, ctx);
 		return;
 	}
@@ -846,7 +828,7 @@ process_cman_msg(cman_handle_t h, void *priv, char *buf, int len,
 		    contexts[m->dest_ctx]->type != MSG_CLUSTER) {
 			/* XXX Work around bug where M_CLOSE is called
 			   on a context which has been destroyed */
-			clulog(LOG_WARNING, "Ignoring M_CLOSE for destroyed "
+			log_printf(LOG_WARNING, "Ignoring M_CLOSE for destroyed "
 			       "context %d\n", m->dest_ctx);
 		} else {
 			queue_for_context(contexts[m->dest_ctx], buf, len);
