@@ -1,21 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2002-2003
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 /** @file
  * Implements rmtab read/write/list handling functions utilized by
  * clurmtabd.
@@ -222,6 +204,7 @@ rmtab_insert(rmtab_node **head, rmtab_node *pre, char *host,
 
 	/* Copy in info */
 	rnew = malloc(sizeof(*rnew));
+	// FIXME: handle failed malloc
 	memset(rnew, 0, sizeof(*rnew));
 	rnew->rn_hostname = strdup(host);
 	rnew->rn_path = strdup(path);
