@@ -708,8 +708,8 @@ event_loop(msgctx_t *localctx, msgctx_t *clusterctx)
 
 	if (signalled) {
 		signalled = 0;
- 
-		dump_internal_state("/tmp/rgmanager-dump");
+
+		dump_internal_state("/var/lib/cluster/rgmanager-dump");
 	}
 
 	while (running && (tv.tv_sec || tv.tv_usec)) {
