@@ -11,8 +11,9 @@
 #include <restart_counter.h>
 #include <reslist.h>
 #include <pthread.h>
+#include <libgen.h>
 #ifndef NO_CCS
-#include <clulog.h>
+#include <logging.h>
 #endif
 
 
@@ -398,7 +399,7 @@ store_resource(resource_t **reslist, resource_t *newres)
 					       newres->r_attrs[x].ra_value
 					       );
 #else 
-					clulog(LOG_ERR,
+					log_printf(LOG_ERR,
                                                "%s attribute collision. "
                                                "type=%s attr=%s value=%s\n",
 					       (newres->r_attrs[x].ra_flags&
@@ -431,7 +432,8 @@ store_resource(resource_t **reslist, resource_t *newres)
    @return		newly allocated pointer to value or NULL if not found.
  */
 char *
-xpath_get_one(xmlDocPtr doc, xmlXPathContextPtr ctx, char *query)
+xpath_get_one(xmlDocPtr __attribute__ ((unused)) doc,
+	      xmlXPathContextPtr ctx, char *query)
 {
 	char *val = NULL, *ret = NULL;
 	xmlXPathObjectPtr obj;
@@ -626,7 +628,11 @@ act_dup(resource_act_t *acts)
 
 /* Copied from resrules.c -- _get_actions */
 void
+#ifndef NO_CCS
 _get_actions_ccs(int ccsfd, char *base, resource_t *res)
+#else
+_get_actions_ccs(int __attribute__((unused)) ccsfd, char *base, resource_t *res)
+#endif
 {
 	char xpath[256];
 	int idx = 0;
@@ -848,7 +854,7 @@ load_resources(int ccsfd, resource_t **reslist, resource_rule_t **rulelist)
 	       		       printf("Error storing %s resource\n",
 				      newres->r_rule->rr_type);
 #else
-	       		       clulog(LOG_ERR,
+	       		       log_printf(LOG_ERR,
 				      "Error storing %s resource\n",
 				      newres->r_rule->rr_type);
 #endif
