@@ -4,12 +4,13 @@
 #include <assert.h>
 #include <libcman.h>
 #include <ccs.h>
-#include <clulog.h>
+#include <logging.h>
 #include <lock.h>
 #include <event.h>
 #include <stdint.h>
 #include <vf.h>
 #include <members.h>
+#include <time.h>
 
 
 /**
@@ -23,6 +24,7 @@ static pthread_mutex_t mi_mutex = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
 static pthread_mutex_t event_queue_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mi_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
+static pthread_cond_t event_queue_cond = PTHREAD_COND_INITIALIZER;
 static pthread_t event_thread = 0;
 static int transition_throttling = 5;
 static int central_events = 0;
@@ -77,7 +79,8 @@ central_events_enabled(void)
   @see eval_groups
  */
 void
-node_event(int local, int nodeID, int nodeStatus, int clean)
+node_event(int local, int nodeID, int nodeStatus,
+	   int __attribute__((unused)) clean)
 {
 	if (!running)
 		return;
@@ -86,20 +89,20 @@ node_event(int local, int nodeID, int nodeStatus, int clean)
 
 		/* Local Node Event */
 		if (nodeStatus == 0) {
-			clulog(LOG_ERR, "Exiting uncleanly\n");
+			log_printf(LOG_ERR, "Exiting uncleanly\n");
 			hard_exit();
 		}
 
 		if (!rg_initialized()) {
 			if (init_resource_groups(0) != 0) {
-				clulog(LOG_ERR,
+				log_printf(LOG_ERR,
 				       "#36: Cannot initialize services\n");
 				hard_exit();
 			}
 		}
 
 		if (shutdown_pending) {
-			clulog(LOG_NOTICE, "Processing delayed exit signal\n");
+			log_printf(LOG_NOTICE, "Processing delayed exit signal\n");
 			running = 0;
 			return;
 		}
@@ -115,7 +118,7 @@ node_event(int local, int nodeID, int nodeStatus, int clean)
 	 * Nothing to do for events from other nodes if we are not ready.
 	 */
 	if (!rg_initialized()) {
-		clulog(LOG_DEBUG, "Services not initialized.\n");
+		log_printf(LOG_DEBUG, "Services not initialized.\n");
 		return;
 	}
 
@@ -138,7 +141,7 @@ node_has_fencing(int nodeid)
 	
 	ccs_desc = ccs_connect();
 	if (ccs_desc < 0) {
-		clulog(LOG_ERR, "Unable to connect to ccsd; cannot handle"
+		log_printf(LOG_ERR, "Unable to connect to ccsd; cannot handle"
 		       " node event!\n");
 		/* Assume node has fencing */
 		return 1;
@@ -157,6 +160,20 @@ node_has_fencing(int nodeid)
 }
 
 
+/* Since the API for groupd is private, use group_tool to find
+   out if we've joined the fence domain */ 
+int
+fence_domain_joined(void)
+{
+	int rv;
+
+	rv = system("group_tool ls fence default &> /dev/null");	
+	if (rv == 0)
+		return 1;
+	return 0;
+}
+
+
 /**
    Quick query to cman to see if a node has been fenced.
  */
@@ -182,27 +199,28 @@ node_fenced(int nodeid)
    Master key.
  */
 int32_t
-master_event_callback(char *key, uint64_t viewno,
+master_event_callback(char __attribute__ ((unused)) *key,
+		      uint64_t __attribute__ ((unused)) viewno,
 		      void *data, uint32_t datalen)
 {
 	event_master_t *m;
 
 	m = data;
 	if (datalen != (uint32_t)sizeof(*m)) {
-		clulog(LOG_ERR, "%s: wrong size\n", __FUNCTION__);
+		log_printf(LOG_ERR, "%s: wrong size\n", __FUNCTION__);
 		return 1;
 	}
 
 	swab_event_master_t(m);
 	if (m->m_magic != EVENT_MASTER_MAGIC) {
-		clulog(LOG_ERR, "%s: wrong size\n", __FUNCTION__);
+		log_printf(LOG_ERR, "%s: wrong size\n", __FUNCTION__);
 		return 1;
 	}
 
-	if (m->m_nodeid == my_id())
-		clulog(LOG_DEBUG, "Master Commit: I am master\n");
+	if (m->m_nodeid == (uint32_t)my_id())
+		log_printf(LOG_DEBUG, "Master Commit: I am master\n");
 	else 
-		clulog(LOG_DEBUG, "Master Commit: %d is master\n", m->m_nodeid);
+		log_printf(LOG_DEBUG, "Master Commit: %d is master\n", m->m_nodeid);
 
 	pthread_mutex_lock(&mi_mutex);
 	if (mi)
@@ -231,7 +249,7 @@ find_master(void)
 	m = member_list();
 	if (vf_read(m, "Transition-Master", &vn,
 		    (void **)(&data), &sz) < 0) {
-		clulog(LOG_ERR, "Unable to discover master"
+		log_printf(LOG_ERR, "Unable to discover master"
 		       " status\n");
 		masterinfo = NULL;
 	} else {
@@ -242,7 +260,7 @@ find_master(void)
 	if (masterinfo && (sz >= sizeof(*masterinfo))) {
 		swab_event_master_t(masterinfo);
 		if (masterinfo->m_magic == EVENT_MASTER_MAGIC) {
-			clulog(LOG_DEBUG, "Master Locate: %d is master\n",
+			log_printf(LOG_DEBUG, "Master Locate: %d is master\n",
 			       masterinfo->m_nodeid);
 			pthread_mutex_lock(&mi_mutex);
 			if (mi)
@@ -304,7 +322,7 @@ event_master(void)
 		master_id = mi->m_nodeid;
 		pthread_mutex_unlock(&mi_mutex);
 		if (memb_online(m, master_id)) {
-			//clulog(LOG_DEBUG, "%d is master\n", mi->m_nodeid);
+			//log_printf(LOG_DEBUG, "%d is master\n", mi->m_nodeid);
 			goto out;
 		}
 	}
@@ -335,7 +353,7 @@ event_master(void)
 	if (vf_write(m, VFF_IGN_CONN_ERRORS | VFF_RETRY,
 		     "Transition-Master", &masterinfo,
 		     sizeof(masterinfo)) < 0) {
-		clulog(LOG_ERR, "Unable to advertise master"
+		log_printf(LOG_ERR, "Unable to advertise master"
 		       " status to all nodes\n");
 	}
 
@@ -354,31 +372,43 @@ void group_event(char *name, uint32_t state, int owner);
   events are on the queue.
  */
 void *
-_event_thread_f(void *arg)
+_event_thread_f(void __attribute__ ((unused)) *arg)
 {
 	event_t *ev;
+	struct timeval now;
+	struct timespec expire;
 	int notice = 0, count = 0;
 
+	/* Event thread usually doesn't hang around.  When it's
+   	   spawned, sleep for this many seconds in order to let
+   	   some events queue up */
+	if (transition_throttling && !central_events) {
+		sleep(transition_throttling);
+	}
+
 	while (1) {
 		pthread_mutex_lock(&event_queue_mutex);
 		ev = event_queue;
-		if (ev)
-			list_remove(&event_queue, ev);
-		else
+		if (!ev && !central_events) {
+			gettimeofday(&now, NULL);
+			expire.tv_sec = now.tv_sec + 5;
+			expire.tv_nsec = now.tv_usec * 1000;
+			pthread_cond_timedwait(&event_queue_cond,
+						&event_queue_mutex,
+						&expire);
+			ev = event_queue;
+		}
+		if (!ev)
 			break; /* We're outta here */
 
-		++count;
-		/* Event thread usually doesn't hang around.  When it's
-	   	   spawned, sleep for this many seconds in order to let
-	   	   some events queue up */
-		if ((count==1) && transition_throttling && !central_events)
-			sleep(transition_throttling);
+		list_remove(&event_queue, ev);
 
+		++count;
 		pthread_mutex_unlock(&event_queue_mutex);
 
 		if (ev->ev_type == EVENT_CONFIG) {
 			/*
-			clulog(LOG_NOTICE, "Config Event: %d -> %d\n",
+			log_printf(LOG_NOTICE, "Config Event: %d -> %d\n",
 			       ev->ev.config.cfg_oldversion,
 			       ev->ev.config.cfg_version);
 			 */
@@ -402,7 +432,7 @@ _event_thread_f(void *arg)
 
 		if (ev->ev_type == EVENT_RG) {
 			/*
-			clulog(LOG_NOTICE, "RG Event: %s %s %d\n",
+			log_printf(LOG_NOTICE, "RG Event: %s %s %d\n",
 			       ev->ev.group.rg_name,
 			       rg_state_str(ev->ev.group.rg_state),
 			       ev->ev.group.rg_owner);
@@ -412,7 +442,7 @@ _event_thread_f(void *arg)
 				    ev->ev.group.rg_owner);
 		} else if (ev->ev_type == EVENT_NODE) {
 			/*
-			clulog(LOG_NOTICE, "Node Event: %s %d %s %s\n",
+			log_printf(LOG_NOTICE, "Node Event: %s %d %s %s\n",
 			       ev->ev.node.ne_local?"Local":"Remote",
 			       ev->ev.node.ne_nodeid,
 			       ev->ev.node.ne_state?"UP":"DOWN",
@@ -426,7 +456,7 @@ _event_thread_f(void *arg)
 				while (!node_fenced(ev->ev.node.ne_nodeid)) {
 					if (!notice) {
 						notice = 1;
-						clulog(LOG_INFO, "Waiting for "
+						log_printf(LOG_INFO, "Waiting for "
 						       "node #%d to be fenced\n",
 						       ev->ev.node.ne_nodeid);
 					}
@@ -434,7 +464,7 @@ _event_thread_f(void *arg)
 				}
 
 				if (notice)
-					clulog(LOG_INFO, "Node #%d fenced; "
+					log_printf(LOG_INFO, "Node #%d fenced; "
 					       "continuing\n",
 					       ev->ev.node.ne_nodeid);
 			}
@@ -449,7 +479,7 @@ _event_thread_f(void *arg)
 	}
 
 	if (!central_events || _master) {
-		clulog(LOG_DEBUG, "%d events processed\n", count);
+		log_printf(LOG_DEBUG, "%d events processed\n", count);
 	}
 	/* Mutex held */
 	event_thread = 0;
@@ -473,6 +503,8 @@ insert_event(event_t *ev)
 
 		pthread_create(&event_thread, &attrs, _event_thread_f, NULL);
         	pthread_attr_destroy(&attrs);
+	} else {
+		pthread_cond_broadcast(&event_queue_cond);
 	}
 	pthread_mutex_unlock (&event_queue_mutex);
 }
