@@ -11,7 +11,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <resgroup.h>
-#include <clulog.h>
+#include <logging.h>
 #include <lock.h>
 #include <rg_locks.h>
 #include <ccs.h>
@@ -121,13 +121,13 @@ svc_report_failure(char *svcName)
 	cluster_member_list_t *membership;
 
 	if (rg_lock(svcName, &lockp) == -1) {
-		clulog(LOG_ERR, "#41: Couldn't obtain lock for RG %s: %s\n",
+		log_printf(LOG_ERR, "#41: Couldn't obtain lock for RG %s: %s\n",
 		       svcName, strerror(errno));
 		return -1;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
-		clulog(LOG_ERR, "#42: Couldn't obtain status for RG %s\n",
+		log_printf(LOG_ERR, "#42: Couldn't obtain status for RG %s\n",
 		       svcName);
 		rg_unlock(&lockp);
 		return -1;
@@ -137,19 +137,18 @@ svc_report_failure(char *svcName)
 	membership = member_list();
 	nodeName = memb_id_to_name(membership, svcStatus.rs_last_owner);
 	if (nodeName) {
-		clulog(LOG_ALERT, "#2: Service %s returned failure "
+		log_printf(LOG_ALERT, "#2: Service %s returned failure "
 		       "code.  Last Owner: %s\n", svcName, nodeName);
 	} else {
-		clulog(LOG_ALERT, "#3: Service %s returned failure "
+		log_printf(LOG_ALERT, "#3: Service %s returned failure "
 		       "code.  Last Owner: %d\n",
 		       svcName, (int)svcStatus.rs_last_owner);
 	}
 
 	free_member_list(membership);
 
-	clulog(LOG_ALERT,
-	       "#4: Administrator intervention required.\n",
-	       svcName, nodeName);
+	log_printf(LOG_ALERT,
+	       "#4: Administrator intervention required.\n");
 
 	return 0;
 }
@@ -173,7 +172,7 @@ rg_lock(char *name, struct dlm_lksb *p)
 int
 _rg_lock_dbg(char *name, struct dlm_lksb *p, char *file, int line)
 {
-	dprintf("rg_lock(%s) @ %s:%d\n", name, file, line);
+	dbg_printf("rg_lock(%s) @ %s:%d\n", name, file, line);
 	return _rg_lock(name, p);
 }
 #endif
@@ -194,7 +193,7 @@ rg_unlock(struct dlm_lksb *p)
 int
 _rg_unlock_dbg(struct dlm_lksb *p, char *file, int line)
 {
-	dprintf("rg_unlock() @ %s:%d\n", file, line);
+	dbg_printf("rg_unlock() @ %s:%d\n", file, line);
 	return _rg_unlock(p);
 }
 #endif
@@ -463,6 +462,8 @@ get_rg_state_local(char *name, rg_state_t *svcblk)
  *                      3 = DO NOT stop service, return RG_EFORWARD
  *			4 = DO NOT stop service, return RG_EAGAIN
  *			5 = DO NOT stop service, return RG_EFROZEN
+ *			6 = DO NOT stop service, mark stopped and return
+ *			    RG_SUCCESS (0)
  */
 int
 svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
@@ -471,7 +472,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 	int ret = 0;
 	
 	if (svcStatus->rs_flags & RG_FLAG_FROZEN) {
-		clulog(LOG_DEBUG, "Service %s frozen.\n", svcName);
+		log_printf(LOG_DEBUG, "Service %s frozen.\n", svcName);
 		return 5;
 	}
 
@@ -489,7 +490,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 	case RG_STATE_STARTING:
 	case RG_STATE_RECOVER:
 	case RG_STATE_MIGRATE:
-		if ((svcStatus->rs_owner != my_id()) &&
+		if ((svcStatus->rs_owner != (uint32_t)my_id()) &&
 		    memb_online(membership, svcStatus->rs_owner)) {
 			/*
 			   Service is running and the owner is online.
@@ -507,7 +508,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 			}
 
 			/* Disable or relocate request here. */
-			clulog(LOG_DEBUG, "Forwarding req. to %s.\n",
+			log_printf(LOG_DEBUG, "Forwarding req. to %s.\n",
 			       memb_id_to_name(membership,
 					       svcStatus->rs_owner));
 			ret = 3;
@@ -515,7 +516,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 		}
 
 		if (svcStatus->rs_owner == 0 ||
-		    (svcStatus->rs_owner == my_id())) {
+		    (svcStatus->rs_owner == (uint32_t)my_id())) {
 			/*
 			   Service is marked as running locally or on
 			   0 (e.g. no member).  Safe
@@ -527,9 +528,10 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 
 		/*
 		   Service is marked as running but node is down.
-		   Doesn't make much sense to stop it.
+		   Doesn't make much sense to stop it - but we need
+		   to mark it stopped
 		 */
-		ret = 2;
+		ret = 6;
 		break;
 
 	case RG_STATE_ERROR:
@@ -538,7 +540,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 			ret = 1;
 			break;
 		}
-		clulog(LOG_DEBUG,
+		log_printf(LOG_DEBUG,
 		       "Not stopping %s: service is failed\n",
 		       svcName);
 		ret = 0;
@@ -555,7 +557,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 	case RG_STATE_DISABLED:
 	case RG_STATE_UNINITIALIZED:
 		if (req == RG_DISABLE) {
-			clulog(LOG_NOTICE,
+			log_printf(LOG_NOTICE,
 			       "Disabling disabled service %s\n",
 			       svcName);
 			ret = 1;
@@ -563,12 +565,12 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 		}
 
 		ret = 2;
-		clulog(LOG_DEBUG, "Not stopping disabled service %s\n",
+		log_printf(LOG_DEBUG, "Not stopping disabled service %s\n",
 		       svcName);
 		break;
 
 	default:
-		clulog(LOG_ERR,
+		log_printf(LOG_ERR,
 		       "#42: Cannot stop RG %s: Invalid State %d\n",
 		       svcName, svcStatus->rs_state);
 		break;
@@ -601,13 +603,13 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 	int ret = 0;
 	
 	if (svcStatus->rs_flags & RG_FLAG_FROZEN) {
-		clulog(LOG_DEBUG, "Service %s frozen.\n", svcName);
+		log_printf(LOG_DEBUG, "Service %s frozen.\n", svcName);
 		return 5;
 	}
 
 	switch(svcStatus->rs_state) {
 	case RG_STATE_FAILED:
-		clulog(LOG_ERR,
+		log_printf(LOG_ERR,
 		       "#43: Service %s has failed; can not start.\n",
 		       svcName);
 		break;
@@ -620,21 +622,21 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 	case RG_STATE_STARTED:
 	case RG_STATE_CHECK:
 	case RG_STATE_STARTING:
-		if (svcStatus->rs_owner == my_id()) {
+		if (svcStatus->rs_owner == (uint32_t)my_id()) {
 		    	/*
 			 * Service is already running locally
-			clulog(LOG_DEBUG,
+			log_printf(LOG_DEBUG,
 			       "RG %s is already running locally\n", svcName);
 			 */
 			ret = 4;
 			break;
 		}
 
-		if (svcStatus->rs_owner != my_id() &&
+		if (svcStatus->rs_owner != (uint32_t)my_id() &&
 		    memb_online(membership, svcStatus->rs_owner)) {
 			/*
 			 * Service is running and the owner is online!
-			clulog(LOG_DEBUG, "RG %s is running on member %s.\n",
+			log_printf(LOG_DEBUG, "RG %s is running on member %s.\n",
 			       svcName,
 			       memb_id_to_name(membership,svcStatus->rs_owner));
 			 */
@@ -650,7 +652,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 				break;
 			}
 
-			clulog(LOG_NOTICE,
+			log_printf(LOG_NOTICE,
 			       "Starting stopped service%s\n",
 			       svcName);
 			ret = 1;
@@ -658,7 +660,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		}
 
 		if (rg_locked()) {
-			clulog(LOG_WARNING, "Not initiating failover of %s: "
+			log_printf(LOG_WARNING, "Not initiating failover of %s: "
 			       "Resource groups locked!\n", svcName);
 			ret = 3;
 			break;
@@ -667,7 +669,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		/*
 		 * Service is running but owner is down -> RG_EFAILOVER
 		 */
-		clulog(LOG_NOTICE,
+		log_printf(LOG_NOTICE,
 		       "Taking over service %s from down member %s\n",
 		       svcName, memb_id_to_name(membership,
 						svcStatus->rs_owner));
@@ -679,7 +681,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		 * Starting failed service...
 		 */
 		if (req == RG_START_RECOVER || central_events_enabled()) {
-			clulog(LOG_NOTICE,
+			log_printf(LOG_NOTICE,
 			       "Recovering failed service %s\n",
 			       svcName);
 			svcStatus->rs_state = RG_STATE_STOPPED;
@@ -689,7 +691,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		}
 
 		/* Don't start, but return success. */
-		clulog(LOG_DEBUG,
+		log_printf(LOG_DEBUG,
 		       "Not starting %s: recovery state\n",
 		       svcName);
 		ret = 2;
@@ -703,7 +705,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 			break;
 		}
 
-		clulog(LOG_NOTICE, "Starting stopped service %s\n",
+		log_printf(LOG_NOTICE, "Starting stopped service %s\n",
 		       svcName);
 		ret = 1;
 		break;
@@ -717,7 +719,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 				break;
 			}
 
-			clulog(LOG_NOTICE,
+			log_printf(LOG_NOTICE,
 			       "Starting disabled service %s\n",
 			       svcName);
 			ret = 1;
@@ -728,12 +730,12 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 			break;
 		}
 
-		clulog(LOG_DEBUG, "Not starting disabled RG %s\n",
+		log_printf(LOG_DEBUG, "Not starting disabled RG %s\n",
 		       svcName);
 		break;
 
 	default:
-		clulog(LOG_ERR,
+		log_printf(LOG_ERR,
 		       "#44: Cannot start RG %s: Invalid State %d\n",
 		       svcName, svcStatus->rs_state);
 		break;
@@ -760,14 +762,14 @@ svc_start(char *svcName, int req)
 	rg_state_t svcStatus;
 
 	if (rg_lock(svcName, &lockp) < 0) {
-		clulog(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#46: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#46: Failed getting status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
@@ -805,7 +807,7 @@ svc_start(char *svcName, int req)
 		svcStatus.rs_restarts = 0;
 
 	if (set_rg_state(svcName, &svcStatus) < 0) {
-		clulog(LOG_ERR,
+		log_printf(LOG_ERR,
 		       "#47: Failed changing service status\n");
 		rg_unlock(&lockp);
 		return RG_EFAIL;
@@ -818,14 +820,14 @@ svc_start(char *svcName, int req)
 			cute values scripts might return */
 
 	if (rg_lock(svcName, &lockp) < 0) {
-		clulog(LOG_ERR, "#74: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#74: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return RG_EFAIL;
 	}
 
 	svcStatus.rs_state = RG_STATE_STARTED;
 	if (set_rg_state(svcName, &svcStatus) != 0) {
-		clulog(LOG_ERR,
+		log_printf(LOG_ERR,
 		       "#75: Failed changing service status\n");
 		rg_unlock(&lockp);
 		return RG_EFAIL;
@@ -833,14 +835,14 @@ svc_start(char *svcName, int req)
 	rg_unlock(&lockp);
        
 	if (ret == 0) {
-		clulog(LOG_NOTICE,
+		log_printf(LOG_NOTICE,
 		       "Service %s started\n",
 		       svcName);
 
 		broadcast_event(svcName, RG_STATE_STARTED, svcStatus.rs_owner,
 				svcStatus.rs_last_owner);
 	} else {
-		clulog(LOG_WARNING,
+		log_printf(LOG_WARNING,
 		       "#68: Failed to start %s; return value: %d\n",
 		       svcName, ret);
 	}
@@ -880,25 +882,25 @@ svc_migrate(char *svcName, int target)
 	free_member_list(membership);
 
 	if (rg_lock(svcName, &lockp) < 0) {
-		clulog(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#46: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#46: Failed getting status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
 
-	if (svcStatus.rs_owner == target) {
+	if (svcStatus.rs_owner == (uint32_t)target) {
 		rg_unlock(&lockp);
 		/* Do not allow migration to its current owner! */
 		return 0;
 	}
 
-	if (svcStatus.rs_owner != my_id()) {
+	if (svcStatus.rs_owner != (uint32_t)my_id()) {
 		rg_unlock(&lockp);
 		return RG_EFORWARD;
 	}
@@ -925,7 +927,7 @@ svc_migrate(char *svcName, int target)
 	svcStatus.rs_transition = (uint64_t)time(NULL);
 
 	if (set_rg_state(svcName, &svcStatus) != 0) {
-		clulog(LOG_ERR,
+		log_printf(LOG_ERR,
 		       "#75: Failed changing service status\n");
 		rg_unlock(&lockp);
 		return RG_EFAIL;
@@ -963,20 +965,20 @@ svc_migrate(char *svcName, int target)
 	   services are ... well, migratable, we can just flip the state
 	   back to 'started' and error checking will fix it later. */
 	if (rg_lock(svcName, &lockp) < 0) {
-		clulog(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return ret;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#46: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#46: Failed getting status for RG %s\n",
 		       svcName);
 		return ret;
 	}
 
-	if (svcStatus.rs_last_owner != my_id() ||
-	    svcStatus.rs_owner != target ||
+	if (svcStatus.rs_last_owner != (uint32_t)my_id() ||
+	    svcStatus.rs_owner != (uint32_t)target ||
 	    svcStatus.rs_state != RG_STATE_MIGRATE) {
 		rg_unlock(&lockp);
 		return ret;
@@ -1080,24 +1082,24 @@ msvc_check_cluster(char *svcName)
 
 	newowner = get_new_owner(svcName);
 	if (newowner < 0) {
-		clulog(LOG_DEBUG, "No other nodes have seen %s\n", svcName);
+		log_printf(LOG_DEBUG, "No other nodes have seen %s\n", svcName);
 		return -1;
 	}
 
 	/* New owner found */
-	clulog(LOG_NOTICE, "Migration: %s is running on %d\n", svcName, newowner);
+	log_printf(LOG_NOTICE, "Migration: %s is running on %d\n", svcName, newowner);
 
 	/* If the check succeeds (returns 0), then flip the state back to
 	   'started' - with a new owner */
 	if (rg_lock(svcName, &lockp) < 0) {
-		clulog(LOG_ERR, "#451: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#451: Unable to obtain cluster lock: %s\n",
 			strerror(errno));
 		return -1;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#452: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#452: Failed getting status for RG %s\n",
 		       svcName);
 		return -1;
 	}
@@ -1107,7 +1109,7 @@ msvc_check_cluster(char *svcName)
 
 	if (set_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#453: Failed setting status for RG %s\n",
+		log_printf(LOG_ERR, "#453: Failed setting status for RG %s\n",
 		       svcName);
 		return -1;
 	}
@@ -1131,14 +1133,14 @@ svc_status(char *svcName)
 	int ret;
 
 	if (rg_lock(svcName, &lockp) < 0) {
-		clulog(LOG_ERR, "#48: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#48: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#49: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#49: Failed getting status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
@@ -1148,7 +1150,7 @@ svc_status(char *svcName)
 		/* Don't check status if the service is frozen */
 		return 0;
 
-	if (svcStatus.rs_owner != my_id())
+	if (svcStatus.rs_owner != (uint32_t)my_id())
 		/* Don't check status for anything not owned */
 		return 0;
 
@@ -1168,7 +1170,8 @@ svc_status(char *svcName)
 
 
 static inline int
-handle_started_status(char *svcName, int ret, rg_state_t *svcStatus)
+handle_started_status(char *svcName, int ret,
+		      rg_state_t __attribute__((unused)) *svcStatus)
 {
 	int newowner;
 
@@ -1183,7 +1186,7 @@ handle_started_status(char *svcName, int ret, rg_state_t *svcStatus)
 	   inline recovery */
 	if (ret & SFL_RECOVERABLE) {
 
-		clulog(LOG_WARNING, "Some independent resources in %s failed; "
+		log_printf(LOG_WARNING, "Some independent resources in %s failed; "
 		       "Attempting inline recovery\n", svcName);
 
 		ret = group_op(svcName, RG_CONDSTOP);
@@ -1192,10 +1195,10 @@ handle_started_status(char *svcName, int ret, rg_state_t *svcStatus)
 		}
 
 		if (ret) {
-			clulog(LOG_WARNING, "Inline recovery of %s failed\n",
+			log_printf(LOG_WARNING, "Inline recovery of %s failed\n",
 			       svcName);
 		} else {
-			clulog(LOG_NOTICE,
+			log_printf(LOG_NOTICE,
 			       "Inline recovery of %s succeeded\n",
 			       svcName);
 			return 0;
@@ -1219,7 +1222,7 @@ handle_migrate_status(char *svcName, int ret, rg_state_t *svcStatus)
 	/* If the check succeeds (returns 0), then flip the state back to
 	   'started' - we now own the service */
 	if (rg_lock(svcName, &lockp) < 0) {
-		clulog(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
 			strerror(errno));
 		return RG_EFAIL;
 	}
@@ -1227,13 +1230,13 @@ handle_migrate_status(char *svcName, int ret, rg_state_t *svcStatus)
 	svcStatus->rs_state = RG_STATE_STARTED;
 	if (set_rg_state(svcName, svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#46: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#46: Failed getting status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
 	rg_unlock(&lockp);
 
-	clulog(LOG_NOTICE, "%s is now running locally\n", svcName);
+	log_printf(LOG_NOTICE, "%s is now running locally\n", svcName);
 
 	return 0;
 }
@@ -1245,7 +1248,7 @@ svc_status_inquiry(char *svcName)
 	rg_state_t svcStatus;
 
 	if (get_rg_state_local(svcName, &svcStatus) != 0) {
-		clulog(LOG_ERR, "Failed getting local status for RG %s\n",
+		log_printf(LOG_ERR, "Failed getting local status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
@@ -1274,20 +1277,20 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 	int old_state;
 
 	if (!rg_quorate()) {
-		clulog(LOG_WARNING, "#69: Unclean %s of %s\n", 
+		log_printf(LOG_WARNING, "#69: Unclean %s of %s\n", 
 		       rg_req_str(req), svcName);
 		return group_op(svcName, RG_STOP);
 	}
 
 	if (rg_lock(svcName, &lockp) == RG_EFAIL) {
-		clulog(LOG_ERR, "#50: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#50: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#51: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#51: Failed getting status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
@@ -1295,9 +1298,19 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 	switch (svc_advise_stop(&svcStatus, svcName, req)) {
 	case 0:
 		rg_unlock(&lockp);
-		clulog(LOG_DEBUG, "Unable to stop RG %s in %s state\n",
+		log_printf(LOG_DEBUG, "Unable to stop RG %s in %s state\n",
 		       svcName, rg_state_str(svcStatus.rs_state));
 		return RG_EFAIL;
+	case 6:
+		/* Mark stopped, but do not do anything */
+		svcStatus.rs_last_owner = svcStatus.rs_owner;
+		svcStatus.rs_owner = 0;
+		svcStatus.rs_state = RG_STATE_STOPPED;
+		if (set_rg_state(svcName, &svcStatus) != 0) {
+			rg_unlock(&lockp);
+			return RG_EFAIL;
+		}
+		/* FALLTHROUGH */
 	case 2:
 		rg_unlock(&lockp);
 		return RG_ESUCCESS;
@@ -1316,7 +1329,7 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 
 	old_state = svcStatus.rs_state;
 
-	clulog(LOG_NOTICE, "Stopping service %s\n", svcName);
+	log_printf(LOG_NOTICE, "Stopping service %s\n", svcName);
 
 	if (recover) 
 	       	svcStatus.rs_state = RG_STATE_ERROR;
@@ -1328,7 +1341,7 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 
 	if (set_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#52: Failed changing RG status\n");
+		log_printf(LOG_ERR, "#52: Failed changing RG status\n");
 		return RG_EFAIL;
 	}
 	rg_unlock(&lockp);
@@ -1337,7 +1350,7 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 
 	if (old_state == RG_STATE_FAILED && newstate == RG_STATE_DISABLED) {
 		if (ret)
-			clulog(LOG_ALERT, "Marking %s as 'disabled', "
+			log_printf(LOG_ALERT, "Marking %s as 'disabled', "
 			       "but some resources may still be allocated!\n",
 			       svcName);
 		_svc_stop_finish(svcName, 0, newstate);
@@ -1356,14 +1369,14 @@ _svc_stop_finish(char *svcName, int failed, uint32_t newstate)
 	struct dlm_lksb lockp;
 
 	if (rg_lock(svcName, &lockp) == RG_EFAIL) {
-		clulog(LOG_ERR, "#53: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#53: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#54: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#54: Failed getting status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
@@ -1379,7 +1392,7 @@ _svc_stop_finish(char *svcName, int failed, uint32_t newstate)
 	svcStatus.rs_owner = 0;
 
 	if (failed) {
-		clulog(LOG_CRIT, "#12: RG %s failed to stop; intervention "
+		log_printf(LOG_CRIT, "#12: RG %s failed to stop; intervention "
 		       "required\n", svcName);
 		newstate = RG_STATE_FAILED;
 	} else if (svcStatus.rs_state == RG_STATE_ERROR) {
@@ -1389,14 +1402,14 @@ _svc_stop_finish(char *svcName, int failed, uint32_t newstate)
 
 	svcStatus.rs_state = newstate;
 
-	clulog(LOG_NOTICE, "Service %s is %s\n", svcName,
+	log_printf(LOG_NOTICE, "Service %s is %s\n", svcName,
 	       rg_state_str(svcStatus.rs_state));
 	//printf("rg state = %s\n", rg_state_str(svcStatus.rs_state));
 
 	svcStatus.rs_transition = (uint64_t)time(NULL);
 	if (set_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#55: Failed changing RG status\n");
+		log_printf(LOG_ERR, "#55: Failed changing RG status\n");
 		return RG_EFAIL;
 	}
 	rg_unlock(&lockp);
@@ -1443,24 +1456,24 @@ svc_fail(char *svcName)
 	rg_state_t svcStatus;
 
 	if (rg_lock(svcName, &lockp) == RG_EFAIL) {
-		clulog(LOG_ERR, "#55: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#55: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return RG_EFAIL;
 	}
 
-	clulog(LOG_DEBUG, "Handling failure request for RG %s\n", svcName);
+	log_printf(LOG_DEBUG, "Handling failure request for RG %s\n", svcName);
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#56: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#56: Failed getting status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
 
 	if ((svcStatus.rs_state == RG_STATE_STARTED) &&
-	    (svcStatus.rs_owner != my_id())) {
+	    (svcStatus.rs_owner != (uint32_t)my_id())) {
 		rg_unlock(&lockp);
-		clulog(LOG_DEBUG, "Unable to disable RG %s in %s state\n",
+		log_printf(LOG_DEBUG, "Unable to disable RG %s in %s state\n",
 		       svcName, rg_state_str(svcStatus.rs_state));
 		return RG_EFAIL;
 	}
@@ -1477,7 +1490,7 @@ svc_fail(char *svcName)
 	svcStatus.rs_restarts = 0;
 	if (set_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#57: Failed changing RG status\n");
+		log_printf(LOG_ERR, "#57: Failed changing RG status\n");
 		return RG_EFAIL;
 	}
 	rg_unlock(&lockp);
@@ -1501,16 +1514,17 @@ _svc_freeze(char *svcName, int enabled)
 	rg_state_t svcStatus;
 
 	if (rg_lock(svcName, &lockp) == RG_EFAIL) {
-		clulog(LOG_ERR, "#55: Unable to obtain cluster lock: %s\n",
+		log_printf(LOG_ERR, "#55: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
 		return RG_EFAIL;
 	}
 
-	clulog(LOG_DEBUG, "Handling %s request for RG %s\n", svcName, enabled?"freeze":"unfreeze");
+	log_printf(LOG_DEBUG, "Handling %s request for RG %s\n", enabled?"freeze":"unfreeze",
+	       svcName);
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
-		clulog(LOG_ERR, "#56: Failed getting status for RG %s\n",
+		log_printf(LOG_ERR, "#56: Failed getting status for RG %s\n",
 		       svcName);
 		return RG_EFAIL;
 	}
@@ -1521,23 +1535,23 @@ _svc_freeze(char *svcName, int enabled)
 	case RG_STATE_DISABLED:
 
 		if (enabled == 1) {
-			clulog(LOG_DEBUG, "Freezing RG %s\n", svcName);
+			log_printf(LOG_DEBUG, "Freezing RG %s\n", svcName);
 			svcStatus.rs_flags |= RG_FLAG_FROZEN;
 		} else {
-			clulog(LOG_DEBUG, "Unfreezing RG %s\n", svcName);
+			log_printf(LOG_DEBUG, "Unfreezing RG %s\n", svcName);
 			svcStatus.rs_flags &= ~RG_FLAG_FROZEN;
 		}
 
 		if (set_rg_state(svcName, &svcStatus) != 0) {
 			rg_unlock(&lockp);
-			clulog(LOG_ERR, "#57: Failed changing RG status\n");
+			log_printf(LOG_ERR, "#57: Failed changing RG status\n");
 			return RG_EFAIL;
 		}
 		break;
 
 	default:
 		rg_unlock(&lockp);
-		return RG_EFAIL;
+		return RG_EAGAIN;
 		break;
 	}
 
@@ -1584,7 +1598,7 @@ svc_start_remote(char *svcName, int request, uint32_t target)
 	/* Open a connection to the other node */
 
 	if (msg_open(MSG_CLUSTER, target, RG_PORT, &ctx, 2)< 0) {
-		clulog(LOG_ERR,
+		log_printf(LOG_ERR,
 		       "#58: Failed opening connection to member #%d\n",
 		       target);
 		return -1;
@@ -1595,15 +1609,15 @@ svc_start_remote(char *svcName, int request, uint32_t target)
 
 	/* Send relocate message to the other node */
 	if (msg_send(&ctx, &msg_relo, sizeof (SmMessageSt)) < 
-	    sizeof (SmMessageSt)) {
-		clulog(LOG_ERR,
+	    (int)sizeof (SmMessageSt)) {
+		log_printf(LOG_ERR,
 		       "#59: Error sending remote-start request to member #%d\n",
 		       target);
 		msg_close(&ctx);
 		return -1;
 	}
 
-	clulog(LOG_DEBUG, "Sent remote-start request to %d\n", (int)target);
+	log_printf(LOG_DEBUG, "Sent remote-start request to %d\n", (int)target);
 
 	/* Check the response */
 	do {
@@ -1618,7 +1632,7 @@ svc_start_remote(char *svcName, int request, uint32_t target)
 		/* Check to see if resource groups are locked for local
 		   shutdown */
 		if (rg_locked()) {
-			clulog(LOG_WARNING,
+			log_printf(LOG_WARNING,
 			       "#XX: Cancelling relocation: Shutting down\n");
 			msg_close(&ctx);
 			return RG_NO;
@@ -1630,7 +1644,7 @@ svc_start_remote(char *svcName, int request, uint32_t target)
 			free_member_list(ml);
 			continue;
 		}
-		clulog(LOG_WARNING,
+		log_printf(LOG_WARNING,
 		       "#XX: Cancelling relocation: Target node down\n");
 		free_member_list(ml);
 		msg_close(&ctx);
@@ -1642,7 +1656,7 @@ svc_start_remote(char *svcName, int request, uint32_t target)
 		 * In this case, we don't restart the service, because the 
 		 * service state is actually unknown to us at this time.
 		 */
-		clulog(LOG_ERR, "#60: Mangled reply from member #%d during RG "
+		log_printf(LOG_ERR, "#60: Mangled reply from member #%d during RG "
 		       "relocate\n", target);
 		msg_close(&ctx);
 		return 0;	/* XXX really UNKNOWN */
@@ -1829,19 +1843,19 @@ handle_relocate_req(char *svcName, int request, int preferred_target,
 			return RG_EFAIL;
 		default:
 			/* deliberate fallthrough */
-			clulog(LOG_ERR,
+			log_printf(LOG_ERR,
 			       "#61: Invalid reply from member %d during"
 			       " relocate operation!\n", target);
 		case RG_NO:
 			/* state uncertain */
 			free_member_list(allowed_nodes);
-			clulog(LOG_CRIT, "State Uncertain: svc:%s "
+			log_printf(LOG_CRIT, "State Uncertain: svc:%s "
 			       "nid:%d req:%s ret:%d\n", svcName,
 			       target, rg_req_str(request), ret);
 			return 0;
 		case 0:
 			*new_owner = target;
-			clulog(LOG_NOTICE, "Service %s is now running "
+			log_printf(LOG_NOTICE, "Service %s is now running "
 			       "on member %d\n", svcName, (int)target);
 			free_member_list(allowed_nodes);
 			return 0;
@@ -1864,7 +1878,7 @@ handle_relocate_req(char *svcName, int request, int preferred_target,
 	 */
 exhausted:
 	if (!rg_locked()) {
-		clulog(LOG_WARNING,
+		log_printf(LOG_WARNING,
 		       "#70: Failed to relocate %s; restarting locally\n",
 		       svcName);
 		if (svc_start(svcName, RG_START_RECOVER) == 0) {
@@ -1975,9 +1989,9 @@ handle_start_req(char *svcName, int req, int *new_owner)
 	 * Keep the state open so the other nodes don't try to start
 	 * it.  This allows us to be the 'root' of a given service.
 	 */
-	clulog(LOG_DEBUG, "Stopping failed service %s\n", svcName);
+	log_printf(LOG_DEBUG, "Stopping failed service %s\n", svcName);
 	if (svc_stop(svcName, RG_STOP_RECOVER) != 0) {
-		clulog(LOG_CRIT,
+		log_printf(LOG_CRIT,
 		       "#13: Service %s failed to stop cleanly\n",
 		       svcName);
 		(void) svc_fail(svcName);
@@ -1997,7 +2011,7 @@ relocate:
 	 * we should relocate the service.
 	 */
 	if (actual_failure)
-		clulog(LOG_WARNING, "#71: Relocating failed service %s\n",
+		log_printf(LOG_WARNING, "#71: Relocating failed service %s\n",
 	       	       svcName);
 	ret = handle_relocate_req(svcName, RG_START_RECOVER, -1, new_owner);
 
@@ -2042,8 +2056,8 @@ handle_start_remote_req(char *svcName, int req)
 
 	/* XXX ok, so we need to say "should I start this if I was the
 	   only cluster member online */
-	for (x = 0; x < membership->cml_count; x++) {
-		if (membership->cml_members[x].cn_nodeid == me)
+	for (x = 0; x < (int)membership->cml_count; x++) {
+		if (membership->cml_members[x].cn_nodeid == (int)me)
 			continue;
 
 		membership->cml_members[x].cn_member = 0;
@@ -2106,7 +2120,7 @@ handle_recover_req(char *svcName, int *new_owner)
 
 	/* Check restart counter/timer for this resource */
 	if (check_restart(svcName) > 0) {
-		clulog(LOG_NOTICE, "Restart threshold for %s exceeded; "
+		log_printf(LOG_NOTICE, "Restart threshold for %s exceeded; "
 		       "attempting to relocate\n", svcName);
 		return handle_relocate_req(svcName, RG_START_RECOVER, -1,
 					   new_owner);
@@ -2163,7 +2177,7 @@ handle_fd_start_req(char *svcName, int request, int *new_owner)
 			ret = RG_EFAIL;
 			goto out;
       		default:
-			clulog(LOG_ERR,
+			log_printf(LOG_ERR,
 	 		       "#6X: Invalid reply [%d] from member %d during"
 	  		       " relocate operation!\n", ret, target);
 	   	}
