@@ -1,27 +1,10 @@
-/*
-  Copyright Red Hat, Inc. 2004
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 #include <message.h>
 #include <resgroup.h>
 #include <rg_locks.h>
 #include <gettid.h>
 #include <rg_queue.h>
 #include <assert.h>
+#include <members.h>
 
 /**
  * Resource thread list entry.
@@ -169,7 +152,7 @@ purge_all(request_t **list)
 	while((curr = *list)) {
 
 		list_remove(list, curr);
-		dprintf("Removed request %d\n", curr->rr_request);
+		dbg_printf("Removed request %d\n", curr->rr_request);
 		if (curr->rr_resp_ctx) {
 			send_response(RG_EABORT, 0, curr);
 			msg_close(curr->rr_resp_ctx);
@@ -195,7 +178,7 @@ resgroup_thread_main(void *arg)
 	rg_inc_threads();
 
 	strncpy(myname, arg, 256);
-	dprintf("Thread %s (tid %d) starting\n",myname,gettid());
+	dbg_printf("Thread %s (tid %d) starting\n",myname,gettid());
 
 	pthread_mutex_init(&my_queue_mutex, NULL);
 	pthread_mutex_lock(&my_queue_mutex);
@@ -244,7 +227,7 @@ resgroup_thread_main(void *arg)
 		ret = RG_FAIL;
 		error = 0;
 
-		dprintf("Processing request %s, resource group %s\n",
+		dbg_printf("Processing request %s, resource group %s\n",
 			rg_req_str(req->rr_request), myname);
 
 		/* find ourselves. */
@@ -265,7 +248,7 @@ resgroup_thread_main(void *arg)
 
 		case RG_ENABLE:
 			if (req->rr_target != 0 &&
-			    req->rr_target != my_id()) {
+			    req->rr_target != (unsigned)my_id()) {
 				error = RG_EFORWARD;
 				ret = RG_NONE;
 				break;
@@ -515,7 +498,7 @@ resgroup_thread_main(void *arg)
 	myself = find_resthread_byname(myname);
 
 	if (!myself) {
-		dprintf("I don't exist...\n");
+		dbg_printf("I don't exist...\n");
 		raise(SIGSEGV);
 	}
 
@@ -540,7 +523,7 @@ resgroup_thread_main(void *arg)
 
 	pthread_mutex_unlock(&reslist_mutex);
 
-	dprintf("RGth %s (tid %d): No more requests"
+	dbg_printf("RGth %s (tid %d): No more requests"
 		"; exiting.\n", myname, gettid());
 
 	/* Thread's outta here */
@@ -739,7 +722,7 @@ rt_enqueue_request(const char *resgroupname, int request,
 	if (ret < 0)
 		return ret;
 
-	dprintf("Queued request for %d for %s\n", request, resgroupname);
+	dbg_printf("Queued request for %d for %s\n", request, resgroupname);
 	
 	return 0;	
 }
