@@ -6,7 +6,7 @@
 #include <list.h>
 #include <restart_counter.h>
 #include <reslist.h>
-#include <clulog.h>
+#include <logging.h>
 #include <members.h>
 #include <assert.h>
 #include <event.h>
@@ -16,7 +16,7 @@
 #include <slang.h>
 #include <sys/syslog.h>
 #include <malloc.h>
-#include <clulog.h>
+#include <logging.h>
 #include <sets.h>
 
 static int __sl_initialized = 0;
@@ -82,7 +82,9 @@ static int
    _user_stop = RG_STOP_USER,		/* From clusvcadm */
    _user_relo = RG_RELOCATE,
    _user_restart = RG_RESTART,
-   _user_migrate = RG_MIGRATE;
+   _user_migrate = RG_MIGRATE,
+   _user_freeze = RG_FREEZE,
+   _user_unfreeze = RG_UNFREEZE;
 
 
 SLang_Intrin_Var_Type rgmanager_vars[] =
@@ -131,6 +133,8 @@ SLang_Intrin_Var_Type rgmanager_vars[] =
 	MAKE_VARIABLE("USER_RELOCATE",	&_user_relo,	SLANG_INT_TYPE, 1),
 	MAKE_VARIABLE("USER_RESTART",	&_user_restart,	SLANG_INT_TYPE, 1),
 	MAKE_VARIABLE("USER_MIGRATE",	&_user_migrate,	SLANG_INT_TYPE, 1),
+	MAKE_VARIABLE("USER_FREEZE",	&_user_freeze,	SLANG_INT_TYPE, 1),
+	MAKE_VARIABLE("USER_UNFREEZE",	&_user_unfreeze,SLANG_INT_TYPE, 1),
 
 	/* Errors */
 	MAKE_VARIABLE("rg_error",	&_rg_err,	SLANG_INT_TYPE, 1),
@@ -244,7 +248,14 @@ sl_service_status(char *svcName)
 		return;
 	}
 
-	state_str = strdup(rg_state_str(svcStatus.rs_state));
+	if (svcStatus.rs_flags & RG_FLAG_FROZEN) {
+		/* Special case: "frozen" is a flag, but user scripts should
+		   treat it as a state. */
+		state_str = strdup(rg_flag_str(RG_FLAG_FROZEN));
+	} else {
+		state_str = strdup(rg_state_str(svcStatus.rs_state));
+	}
+
 	if (!state_str) {
 		SLang_verror(SL_RunTime_Error,
 			     "%s: Failed to duplicate state of %s",
@@ -263,6 +274,21 @@ sl_service_status(char *svcName)
 }
 
 
+/* These can be done by the master node */
+int
+sl_service_freeze(char *svcName)
+{
+	return svc_freeze(svcName);
+}
+
+
+int
+sl_service_unfreeze(char *svcName)
+{
+	return svc_unfreeze(svcName);
+}
+
+
 /**
   (nofailback, restricted, ordered, nodelist) = service_domain_info(svcName);
  */
@@ -556,24 +582,12 @@ push_int_array(int *stuff, int len)
 void
 sl_nodes_online(void)
 {
-	int i, *nodes, nodecount = 0;
+	int x, *nodes = NULL, nodecount = 0;
 
-	cluster_member_list_t *membership = member_list();
-	if (!membership)
-		return;
-	nodes = malloc(sizeof(int) * membership->cml_count);
-	if (!nodes)
+	x = member_online_set(&nodes, &nodecount);
+	if (x < 0 || !nodes || !nodecount)
 		return;
 
-	nodecount = 0;
-	for (i = 0; i < membership->cml_count; i++) {
-		if (membership->cml_members[i].cn_member &&
-		    membership->cml_members[i].cn_nodeid != 0) {
-			nodes[nodecount] = membership->cml_members[i].cn_nodeid;
-			++nodecount;
-		}
-	}
-	free_member_list(membership);
 	push_int_array(nodes, nodecount);
 	free(nodes);
 }
@@ -766,7 +780,7 @@ array_to_string(char *buf, int buflen, int *array, int arraylen)
 
 /**
   Start at the end of the arg list and work backwards, prepending a string.
-  This does not support standard clulog / printf formattting; rather, we 
+  This does not support standard log_printf / printf formattting; rather, we 
   just allow integers / strings to be mixed on the stack, figure out the
   type, convert it to the right type, and prepend it on to our log message
 
@@ -775,7 +789,7 @@ array_to_string(char *buf, int buflen, int *array, int arraylen)
      ...
      LOG_EMERG
 
-  This matches up with clulog / syslog mappings in the var table; the above
+  This matches up with log_printf / syslog mappings in the var table; the above
   are constants in the S/Lang interpreter.  Any number of arguments may
   be provided.  Examples are:
 
@@ -789,7 +803,7 @@ array_to_string(char *buf, int buflen, int *array, int arraylen)
 
  */
 void
-sl_clulog(int level)
+sl_log_printf(int level)
 {
 	int t, nargs, len;
 	//int level;
@@ -854,7 +868,7 @@ sl_clulog(int level)
 #if 0
 	printf("<%d> %s\n", level, &logbuf[remain]);
 #endif
-	clulog(level, &logbuf[remain]);
+	log_printf(level, "%s", &logbuf[remain]);
 	return;
 }
 
@@ -863,56 +877,56 @@ sl_clulog(int level)
 void
 sl_log_debug(void)
 {
-	sl_clulog(LOG_DEBUG);
+	sl_log_printf(LOG_DEBUG);
 }
 
 
 void
 sl_log_info(void)
 {
-	sl_clulog(LOG_INFO);
+	sl_log_printf(LOG_INFO);
 }
 
 
 void
 sl_log_notice(void)
 {
-	sl_clulog(LOG_NOTICE);
+	sl_log_printf(LOG_NOTICE);
 }
 
 
 void
 sl_log_warning(void)
 {
-	sl_clulog(LOG_WARNING);
+	sl_log_printf(LOG_WARNING);
 }
 
 
 void
 sl_log_err(void)
 {
-	sl_clulog(LOG_ERR);
+	sl_log_printf(LOG_ERR);
 }
 
 
 void
 sl_log_crit(void)
 {
-	sl_clulog(LOG_CRIT);
+	sl_log_printf(LOG_CRIT);
 }
 
 
 void
 sl_log_alert(void)
 {
-	sl_clulog(LOG_ALERT);
+	sl_log_printf(LOG_ALERT);
 }
 
 
 void
 sl_log_emerg(void)
 {
-	sl_clulog(LOG_EMERG);
+	sl_log_printf(LOG_EMERG);
 }
 
 
@@ -936,6 +950,10 @@ SLang_Intrin_Fun_Type rgmanager_slang[] =
 	MAKE_INTRINSIC_0("service_start", sl_start_service, SLANG_INT_TYPE),
 	MAKE_INTRINSIC_S("service_status", sl_service_status,
 			 SLANG_VOID_TYPE),
+	MAKE_INTRINSIC_S("service_freeze", sl_service_freeze,
+			 SLANG_INT_TYPE),
+	MAKE_INTRINSIC_S("service_unfreeze", sl_service_unfreeze,
+			 SLANG_INT_TYPE),
 
 	/* Node list manipulation */
 	MAKE_INTRINSIC_0("union", sl_union, SLANG_VOID_TYPE),
@@ -967,7 +985,7 @@ rgmanager_slang_error_hook(char *errstr)
 	/* Don't just send errstr, because it might contain
 	   "%s" for example which would result in a crash!
 	   plus, we like the newline :) */
-	clulog(LOG_ERR, "[S/Lang] %s\n", errstr);
+	log_printf(LOG_ERR, "[S/Lang] %s\n", errstr);
 }
 
 
@@ -1015,7 +1033,7 @@ do_slang_run(const char *file, const char *script)
 		ret = SLang_load_string((char *)script);
 
 	if (ret < 0) {
-		clulog(LOG_ERR, "[S/Lang] Script Execution Failure\n");
+		log_printf(LOG_ERR, "[S/Lang] Script Execution Failure\n");
 		SLang_restart(1);
 	}
 
