@@ -1,28 +1,5 @@
 #!/bin/bash
 
-#
-#  Copyright Red Hat, Inc. 2006
-#
-#  This program is free software; you can redistribute it and/or modify it
-#  under the terms of the GNU General Public License as published by the
-#  Free Software Foundation; either version 2, or (at your option) any
-#  later version.
-#
-#  This program is distributed in the hope that it will be useful, but
-#  WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; see the file COPYING.  If not, write to the
-#  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-#  MA 02139, USA.
-#
-#
-#  Author(s):
-#	Marek Grac (mgrac at redhat.com)
-#
-
 export LC_ALL=C
 export LANG=C
 export PATH=/bin:/sbin:/usr/bin:/usr/sbin
@@ -123,8 +100,6 @@ generate_config_file()
 
 start()
 {
-	declare ccs_fd;
-	
 	clog_service_start $CLOG_INIT
 
 	create_pid_directory
@@ -138,14 +113,8 @@ start()
 	fi
 	clog_looking_for $CLOG_INIT "IP Addresses"
 
-        ccs_fd=$(ccs_connect);
-        if [ $? -ne 0 ]; then
-		clog_looking_for $CLOG_FAILED_CCS
-                return $OCF_ERR_GENERIC
-        fi
-
-        get_service_ip_keys "$ccs_fd" "$OCF_RESKEY_service_name"
-        ip_addresses=`build_ip_list "$ccs_fd"`
+        get_service_ip_keys "$OCF_RESKEY_service_name"
+        ip_addresses=`build_ip_list`
 
 	if [ -z "$ip_addresses" ]; then
 		clog_looking_for $CLOG_FAILED_NOT_FOUND "IP Addresses"
